﻿function Get-ipxIP {
<#
    .Synopsis
        Ermittelt die IP-Adresse eines Rechners.
    .Description
        Das Cmdlet ermittelt die IP-Adresse eines Rechners anhand des DNS-Eintrags am zugehörigen DNS-Controller.
        Wird kein Host (-computername) angegeben, ist der Vorgabewert der lokale Host. Es wird die IPv4-Adresse ermittelt, per switch Paameter ist IPv6 möglich.
    .Parameter computername
        Gibt einen alternativen Computernamen an.
    .Parameter IPv6
        Switch Parameter, es wird die IPv6 anstatt der v4 ermittelt.
    .Example
        Get-ipxIP
        Ermittelt die lokale IP-Adresse.
    .Example
        Get-ipxIP -ipv6
        Ermittelt die lokale IPv6-Adresse.
    .Example
        Get-ipxIP -computername firstDC
        Ermittelt die IP-Adresse des fiktiven ersten Domänencontrollers mit Namen firstDC.
    .Example
        $computerlist = Get-AdComputer -filter *
        foreach ($element in $computerlist) { Get-ipxIP -computername $_}
        Fragt im ersten Schritt die Computer der lokalen Domäne ab und ermittelt danach in einer Schleife alle zugehörigen IP-Adressen.
        Get-ADComputer funktioniert ab Server 2008R2 auf dem DC oder den zughörigen Clients mit RSAT.
    .Inputs
        System.String computername
        System.SwitchParameter ipv6
    .Outputs
        System.PSCustomObject
    .Notes
        Version: 1.0.0.0
        Datum: 23.2.2015
    .Component
        System.Net.DNS
#>
param (
    [Parameter(Mandatory=$false,HelpMessage='IP eines Rechnernamens vom DNS abfragen, ohne Eingabe der lokale Rechner')]
    [string]$computername,
    [Parameter(Mandatory=$false,HelpMessage='Switch Parameter für IPv6 anstatt IPv4')]
    [switch]$ipv6
)
    if ($ipv6 ) {
        [System.Net.Dns]::GetHostAddresses($computername) | 
        Where-Object { $_.AddressFamily -eq 'InterNetworkV6' } | 
        Select-Object -Property IPAddressToString
    }
    else {
        [System.Net.Dns]::GetHostAddresses($computername) | 
        Where-Object { $_.AddressFamily -eq 'InterNetwork' } | 
        Select-Object -Property IPAddressToString
    }
}

function Get-ipxSymLinkEvaluation {
    <#
        .Synopsis
            Gibt den Status der symbolischen Linkverfolgung in der Registry an.
        .Description
            Prüft die gesetzten Werte für symbolische Linkverfolgung in der Registry und gibt sie einzeln aus.
        .Inputs
            None
        .Outputs
            System.Array
        .Notes
            Version: 1.0.0.0
            Datum 23.2.2015
            Eventuell neu implementieren um Objekte zu erzeugen
        .Component
            None
    #>

    param (

    )
    Write-Host "Gibt die erlaubten symbolischen Linkverknüpfungen aus der Registry aus." -ForegroundColor Green
    Write-Host "0 bedeutet nicht erlaubt (false), 1 bedeutet erlaubt (true)" -ForegroundColor Green
    [string]$path = 'HKLM:\SYSTEM\CurrentControlSet\Control\FileSystem'
    'Local To Local: {0}' -f ((Get-ItemProperty -Path $path).SymlinkLocalToLocalEvaluation)
    'Local To Remote: {0}' -f ((Get-ItemProperty -Path $path).SymlinkLocalToRemoteEvaluation)
    'Remote To Local: {0}' -f ((Get-ItemProperty -Path $path).SymlinkRemoteToLocalEvaluation)
    'Remote To Remote: {0}' -f ((Get-ItemProperty -Path $path).SymlinkRemoteToRemoteEvaluation)
}

function Enable-ipxSymLinkEvaluation {
    <#
        .Synopsis
            Erlaubt alle symbolischen Linktypen in über die Registry.
        .Description
            Prüft die aktuelle Session auf Adminrechte und erlaubt alle symbolischen Linktypen in über die Registry.
        .Inputs
            None
        .Outputs
            None
        .Notes
            Version: 1.0.0.0
            Datum 23.2.2015
            Eventuell neu implementieren um mit Objekten zu arbeiten
        .Component
            None
    #>

    param (

    )
    if (  !(Test-ipxAdminRights) ) {
        Write-Host "Ohne Administratorrechte keine Änderung der Registry möglich!" -ForegroundColor Red
        break;
    }
    [string]$path = 'HKLM:\SYSTEM\CurrentControlSet\Control\FileSystem'
    [int]$value = 1
    Set-ItemProperty -Path $path -Name SymlinkLocalToLocalEvaluation -Value $value
    Set-ItemProperty -Path $path -Name SymlinkLocalToRemoteEvaluation -Value $value
    Set-ItemProperty -Path $path -Name SymlinkRemoteToLocalEvaluation -Value $value
    Set-ItemProperty -Path $path -Name SymlinkRemoteToRemoteEvaluation -Value $value
}

function Disable-ipxSymLinkEvaluation {
    <#
        .Synopsis
            Unterbindet alle symbolischen Linktypen in über die Registry.
        .Description
            Prüft die aktuelle Session auf Adminrechte und unterbindet alle symbolischen Linktypen in über die Registry.
        .Inputs
            None
        .Outputs
            None
        .Notes
            Version: 1.0.0.0
            Datum 23.2.2015
            Eventuell neu implementieren um mit Objekten zu arbeiten
        .Component
            None
    #>

    param (

    )
    if (  !(Test-ipxAdminRights) ) {
        Write-Host "Ohne Administratorrechte keine Änderung der Registry möglich!" -ForegroundColor Red
        break;
    }
    [int]$value = 0
    [string]$path = 'HKLM:\SYSTEM\CurrentControlSet\Control\FileSystem'
    Set-ItemProperty -Path $path -Name SymlinkLocalToLocalEvaluation -Value $value
    Set-ItemProperty -Path $path -Name SymlinkLocalToRemoteEvaluation -Value $value
    Set-ItemProperty -Path $path -Name SymlinkRemoteToLocalEvaluation -Value $value
    Set-ItemProperty -Path $path -Name SymlinkRemoteToRemoteEvaluation -Value $value
}

function Install-ipxDotNet35 {
    <#
        .Synopsis
            Installiert das .net 3.5 nach.
        .Description
            Ab Windows 8/Server 2012 relevant, wird das .net Framework 3.5 vom Installationsmedium nachinstalliert.
            Dafür wird der sxs-Ordner abgefragt und das Feature über DISM installiert.
            Das Installationsmedium kann als gemountete ISO oder DVD genutzt werden.
        .Inputs
            None
        .Outputs
            None
        .Notes
            Version: 1.1.0.0
            Datum: 1.6.2016
        .Component
            None
    #>

     [string]$Source = Get-ipxFoldername -rootpath c: -description "Sources\SXS Ordner vom Windows Install Ordner"
    DISM /Online /Enable-Feature /FeatureName:NetFx3 /LimitAccess /All /Source:$Source

}

function Get-ipxVMState {
    <#
        .Synopsis
            Fragt den Zustand einer Hyper-V-Maschien ab.
        .Description
            Fragt den Zustand einer Hyper-V-Maschien ab und gibt ihn zurück.
            Dient aktuell für die VeeamZip-Sicherung auf dem lokalen Server.
        .Parameter Name
            Name der virtuallen Maschine, aucha us der Pipeline
        .Inputs
            Name As System.String
        .Outputs
            System.String (State = "Running", "Stopped")
        .Notes
            Version 1.0.0.0 (1.10.2015)
        .Component
            Powershell Hyper-V Module
            lokaler Hyper-V-Host
    #>

    param (
        [Parameter(Mandatory,HelpMessage='Name der virtuellen Maschine',ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true)]
        [string]$Name = ''
    )

    process {
        (Get-VM -Name $Name).State
    }
}

function Stop-ipxVMForVBR {
    <#
        .Synopsis
            Stoppt eine laufende virtuelle Hyper-V-Maschine sofern sie läuft.
        .Description
            Stoppt eine laufende virtuelle Hyper-V-Maschine sofern sie läuft.
            Dient aktuell für die VeeamZip-Sicherung auf dem lokalen Server.
        .Parameter Name
            Name der zu stoppenden virtuellen Maschine
        .Inputs
            Name As System.String
        .Outputs
            Nothing
        .Notes
            Version 1.0.0.0 (1.10.2015)
        .Component
            Powershell Hyper-V Module
            lokaler Hyper-V-Host
    #>

    param (
        [Parameter(Mandatory,HelpMessage='Name der virtuellen Maschine',ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true)]
        [string]$Name = ''
    )
        if ( (Get-ipxVMState -Name $Name) -eq "Running" ) {
            Stop-VM -Name $Name
        }
}

function Get-ipxDotNetVersion {
    <#
        .Synopsis
            Ermittelt die installierte .net-Version
        .Description
            Liest die installierte .net-Version aus der lokalen Registry aus und gibt sie zurück.
            Unterstützt werden 4.5.x und 4.6.x
        .Example 
            Get-ipxDotNetVersion
            Liefert die installierte .net-Version des eigenen Rechners im Format:
            PSChildName Version   Release Product
            ----------- -------   ------- -------
            Client	  4.6.01586	  394802	4.6.2	
            Full	  4.6.01586	  394802	4.6.2	
            Client	  4.0.0.0			

        .Inputs
            None
        .Outputs
            System.Array
        .Notes
            Version: 1.0.0.0
            Datum: 9.1.2017
            Im Internet gefunden und implementiert.
        .Component
            None
    #>

    Get-Childitem 'HKLM:\SOFTWARE\Microsoft\NET Framework Setup\NDP' -recurse |
    Get-ItemProperty -name Version,Release -EA 0 |
    Where { $_.PSChildName -match '^(?!S)\p{L}'} |
    Select PSChildName, Version, Release, @{
      name="Product"
      expression={
          switch -regex ($_.Release) {
            "378389" { [Version]"4.5" }
            "378675|378758" { [Version]"4.5.1" }
            "379893" { [Version]"4.5.2" }
            "393295|393297" { [Version]"4.6" }
            "394254|394271" { [Version]"4.6.1" }
            "394802|394806" { [Version]"4.6.2" }
            {$_ -gt 394806} { [Version]"Undocumented 4.6.2 or higher, please update script" }
          }
        }
    }

}

function Remove-ipxWindowsUpdates {
    <#
        .Synopsis
            Dummy Syn
        .Description
            Dummy Desc
        .Parameter Dummy
            Dummy Par
        .Example 
            Dummy
        .Inputs
            Dummy In
        .Outputs
            Dummy Out
        .Notes
            Dummy Notes
        .Component
            Dummy Com
    #>

    param (

    )
    Get-Date
    dism /online /Cleanup-Image /StartComponentCleanup
    Get-Date
    dism /online /Cleanup-Image /StartComponentCleanup /ResetBase
    Get-Date
}
