﻿function Mount-ipxSQLDatabase {
<#
    .Synopsis
        MS SQL Datenbank an Instanz anhängen
    .Description
        Es werden eine, mehrere oder alle Datenbanken im angegebenen Ordner an die gesetzte Instanz angehängt.
        Sofern sich Systemdatenbanken im Ordner befinden werden diese ausgelassen.
        Datenbank und Logfile müssen sich im selben Verzeichnis befinden und dem üblichen Namensschema DB.mdf und MD_log.ldf entsprechen.
    .Parameter Instance
        Die SQL Instanz angegeben durch Server\Instanz.
    .Parameter Namelist
        Namen der anzuhängenden Datenbanken, einzelner Name oder Array Name1,Name2,Name3.
    .Parameter Path
        Vollständiger Ordnerpfad in dem die Datenbanken gespeichert werden.
    .Parameter AttachAll
        Alle Datenbanken im gewählten Ordner exklusive Systemdatenbanken anhängen.
    .Parameter Recurse
        Es werden rekursiv die Unterordner im angegebenen Pfad nach Datenbanken durchsucht und angehängt.
    .Example
        Mount-ipxSQLDatabase -Instance Server\Instanz -path D:\Database -Namelist DB01
        Hängt die Datenbank DB01 an die Instanz.
    .Example
        Mount-ipxSQLDatabase -Instance Server\Instanz -path D:\Database -Namelist DB01,DB02
        Hängt die Datenbanken DB01 und DB02 an die Instanz.
    .Example
        Mount-ipxSQLDatabase -Instance Server\Instanz -path D:\Database -MountAll
        Hängt alle Datenbanken unter D:\Database an die Instanz, exklusive der Systemdatenbanken.
    .Example
        Mount-ipxSQLDatabase -Instance Server\Instanz -path D:\Database -MountAll -Recurse
        Hängt alle Datenbanken in allen Unterordnern des Ordners an die Instanz, exklusive der Systemdatenbanken.
    .Inputs
        System.String Instance, Path, Namelist als Array
    .Outputs
        None
    .Notes
        Version 1.0.0.0
        Stand 7.6.2016
    .Component
        SQLPS (Powershell SQL Modul)
#>
    

    param (
    [Parameter(Mandatory,HelpMessage='Datenbankinstanz')]
    [string]$Instance,
    [Parameter(HelpMessage='Datenbankname',ParameterSetName='Namelist')]
    [string[]]$Namelist,
    [Parameter(Mandatory,HelpMessage='Pfad zum Dateiordner')]
    [string]$Path,
    [Parameter(HelpMessage='Alle Datenbanken anhängen',ParameterSetName='All')]
    [switch]$MountAll,
    [Parameter(HelpMessage='Rekursive Suche')]
    [switch]$Recurse
    )

    $Parameterset = @{
        Path = $Path
        Recurse = $Recurse
        Filter = "*.mdf"
    }
    $Checklist = Get-ChildItem @Parameterset
    
    $Array = @()
        If ( $MountAll ) {
            Foreach ( $Element in $Checklist ) {
                if ( -not( $Element.BaseName -eq "master" -or $Element.BaseName -eq "model" -or $Element.BaseName -eq "msdbdata" -or $Element.BaseName -eq "tempdb")) {
                    $obj = New-Object psobject
                    $obj | Add-Member NoteProperty BaseName $Element.BaseName.ToString()
                    $obj | Add-Member NoteProperty Databasefile $Element.FullName
                    $obj | Add-Member NoteProperty Logfile  ($Element.Directory.ToString() + "\" + $Element.BaseName.ToString() + "_log.ldf")
                    $Array += $obj
                }
            }
        }
        Else {
            Foreach ( $Name in $Namelist ) {
                Foreach ( $Element in $Checklist ) {
                    If ( $Element.BaseName -eq $Name ) {
                        $obj = New-Object psobject
                        $obj | Add-Member NoteProperty BaseName $Element.BaseName.ToString()
                        $obj | Add-Member NoteProperty Databasefile $Element.FullName
                        $obj | Add-Member NoteProperty Logfile  ($Element.Directory.ToString() + "\" + $Element.BaseName.ToString() + "_log.ldf")
                        $Array += $obj
                    }
                }
            }
        }

        $TotalNumber = $Array.Length
        $Counter = 0

        foreach ($Element in $Array ) {
        $DBName = $Element.BaseName
        $MDF = $Element.Databasefile
        $LDF = $Element.Logfile
$SQLCommand = @"
CREATE DATABASE $DBName ON (FILENAME = '$MDF'),(FILENAME = '$LDF')FOR ATTACH;
"@
        Write-Progress -Activity $DBName -PercentComplete ($Counter/$TotalNumber*100) 
        Invoke-Sqlcmd -ServerInstance $Instance -Query $SQLCommand
        $Counter++
        }
}

function Dismount-ipxSQLDatabase {
    <#
        .Synopsis
            MS SQL Datenbank einer Instanz trennen
        .Description
            Es werden eine, mehrere oder alle Datenbanken der gesetzten Instanz werden getrennt.
            Die Systemdatenbanken werden nicht getrennt.
            Wie beim Trennen im Management Studio oder TSQL verbleiben die getrennten Datenbanken im Ordner.
        .Parameter Instance
            Die SQL Instanz angegeben durch Server\Instanz.
        .Parameter Namelist
            Namen der zu trennenden Datenbanken, einzelner Name oder Array Name1,Name2,Name3.
        .Parameter DetachAll
            Alle Datenbanken der gewählten Instanz trennen, exklusive der Systemdatenbanken.
        .Example
            Dismount-ipxSQLDatabase -Instance Server\Instanz -Namelist DB01
            Trennt die Datenbank DB01.
        .Example
            Dismount-ipxSQLDatabase -Instance Server\Instanz -Namelist DB01,DB02
            Trennt die Datenbanken DB01 und DB02.
        .Example
            Dismount-ipxSQLDatabase -Instance Server\Instanz -DismountAll
            Trennt alle Datenbanken der Instanz, exklusive der Systemdatenbanken.
        .Inputs
            System.String Instance, Namelist als Array
        .Outputs
            None
        .Notes
            Version 1.0.0.0
            Stand 7.6.2016
        .Component
            SQLPS (Powershell SQL Modul)
    #>

    param (
    [Parameter(Mandatory=$true,HelpMessage='Datenbankinstanz')]
    [string]$Instance,
    [Parameter(HelpMessage='Datenbankname',ParameterSetName='Namelist')]
    [string[]]$Namelist,
    [Parameter(HelpMessage='Alle Datenbanken trennen',ParameterSetName='All')]
    [switch]$DismountAll
    )
    $Checklist = @()
    If ( $DismountAll ) {
        $Checklist = (Get-SqlDatabase -ServerInstance $Instance | Where-Object { $_.IsSystemObject -eq $false } )
            
    }
    Else {
        foreach ( $Element in $Namelist ) {
            $Checklist += (Get-SqlDatabase -ServerInstance $Instance -Name $Element)
        }
    }

    $TotalNumber = $Checklist.Length
    $Counter = 0
    
    foreach ( $Element in $Checklist ) {
    $DBName = $Element.Name
$SQLCommand = @"
USE [master]
GO
ALTER DATABASE $DBName
SET SINGLE_USER WITH ROLLBACK IMMEDIATE
GO
sp_detach_db $DBName;
"@
    Write-Progress -Activity $DBName -PercentComplete ($Counter/$TotalNumber*100)
    Invoke-Sqlcmd -ServerInstance $Instance -Query $SQLCommand
    $Counter++
    }
    
}

function Backup-ipxSQLDatabase {
    <#
        .Synopsis
            MS SQL Datenbanksicherung als .bak in einen Ordner
        .Description
            Sichert eine, mehrere oder alle Produktiv- und Systemdatenbanken exklusive der tempdb in einen Ordner.
            Wird -WithDate verwendet, erstellt das Cmdlet einen Unterordner mit aktuellem Datum als Sicherungsziel.
            Aktuell werden nur vollständige Sicherungen unterstützt.
        .Parameter Instance
            Die SQL Instanz angegeben durch Server\Instanz.
        .Parameter Namelist
            Namen der zu sichernden Datenbanken, einzelner Name oder Array Name1,Name2,Name3.
        .Parameter Path
            Vollständiger Ordnerpfad in dem die Sicherungen gespeichert werden.
        .Parameter WithDate
            Erstellt einen Unterordner im Sicherungspfad nach dem Schema yyMMdd
        .Parameter BackupAll
            Sichert alle Datenbanken der Instanz (Produktiv und System) exklusive der tempdb.
        .Parameter ExcludeSystemDB
            Sichert alle Produktivdatenbanken ohne die Systemdatenbanken.
        .Example 
            Backup-ipxSQLDatabase -Instance server\instanz01 -Namelist DB01 -path C:\Backup
            Sichert die Datenbank DB01 unter C:\Backup.
        .Example 
            Backup-ipxSQLDatabase -Instance server\instanz01 -Namelist DB01 -path C:\Backup -WithDate
            Sichert die Datenbank DB01 unter C:\Backup\160607 wenn zur Ausführung der 7.6.2016 wäre.
        .Example 
            Backup-ipxSQLDatabase -Instance server\instanz01 -Namelist DB01,DB02,msdb,master -path C:\Backup
            Sichert die Produktivdatenbanken DB01 und DB02 sowie die SystemDatenbanken msdb und master unter C:\Backup.
        .Example 
            Backup-ipxSQLDatabase -Instance server\instanz01 -BackupAll -path C:\Backup
            Sichert alle Datenbanken unter C:\Backup
            Wird zusätzlich -WithDate verwendet, wird ein Unterordner erstellt.
        .Example 
            Backup-ipxSQLDatabase -Instance server\instanz01 -BackupAll -ExcludeSystemDB -path C:\Backup
            Sichert alle Produktivdatenbanken unter C:\Backup, Systemdatenbanken werden ignoriert.
        .Inputs
            System.String Instanz, Datenbanknamen (optional)
        .Outputs
            System.IO.File Backups als .bak
        .Notes
            Version 1.0.0.0
            Stand 7.6.2016
        .Component
            SQLPS (Powershell SQL Modul)
    #>

    param (
    [Parameter(Mandatory=$true,HelpMessage='Datenbankinstanz als Server\Instanz')]
    [string]$Instance,
    [Parameter(HelpMessage='Namen der zu sichernden Datenbanken',ParameterSetName='Namelist')]
    [string[]]$Namelist,
    [Parameter(Mandatory=$true,HelpMessage='Sicherungsordner in dem die Backups abgelegt werden')]
    [string]$Path,
    [Parameter(HelpMessage='Erstellt einen Unterordner mit aktuellem Datum im Sicherungsordner')]
    [switch]$WithDate,
    [Parameter(HelpMessage='Sichert alle Produktiv- und Systemdatenbanken der Instanz außer tempdb',ParameterSetName='All')]
    [switch]$BackupAll,
    [Parameter(HelpMessage='Sichert alle Produktivdatenbanken, aber nicht die Systemdatenbanken',ParameterSetName='All')]
    [switch]$ExcludeSystemDB
    )
    If ( $WithDate ) { $Path += "\" + (Get-Date -Format yyMMdd) }
    If ( !(Test-Path -Path $Path) ) { New-Item -Path $Path -ItemType Directory }

    $Checklist = @()    
    If ( $BackupAll ) {
        If ( $ExcludeSystemDB ) {
            $Checklist = (Get-SqlDatabase -ServerInstance $Instance | Where-Object { $_.IsSystemObject -eq $false } )
        }
        Else {
            $Checklist = ( Get-SqlDatabase -ServerInstance $Instance | Where-Object { $_.Name -ne "tempdb" } )
        }
    }

    Else {
        foreach ($Element in $Namelist ) {
            $Checklist += (Get-SqlDatabase -ServerInstance $Instance -Name $Element)
        }
    }

    $TotalNumber = $Checklist.Length
    $Counter = 0

    foreach ( $Element in $Checklist ) {
        $DBName = $Element.Name
        Write-Progress -Activity $DBName -PercentComplete ($Counter/$TotalNumber*100)
        Backup-SqlDatabase -ServerInstance $Instance -Database $DBName -BackupFile $Path\$DBName.bak
        $Counter++
    }   
}