﻿function Get-ipxLastWriteTime {
    <#
        .Synopsis
            Filtert Dateien aufgrund ihres Alters und gibt sie aus.
        .Description
            Es werden zunächst Maximal- und Minimalalter der Dateien abgefragt.
            Danach muss der zu prüfende Ordner und eine CSV-Datei für den Logfile angegeben werden.
            Die Funktion ermittelt die relevanten Dateiinformationen und erstellt die CSV-Datei.
            Der Fortschritt wird als aktueller Ordner mit ausgegeben.
        .Inputs
            None
        .Outputs
            System.IO.File logfile *.csv
        .Notes
            Version: 1.0.0.0
            Datum: 23.2.2015
            Muss vollständig neu gebaut werden.
        .Component
            None
    #>

    $oldest = (Get-Date).AddDays(-(Read-Host "Maximalalter in Tagen"))
    $newest = (Get-Date).AddDays(-(Read-Host "Mindestalter in Tagen"))
    if ( $oldest -gt $newest) {
        Write-Host "Das Minimalalter darf nicht vor dem Maximalalter liegen." -ForegroundColor red
        break;
    }
    else {
        Write-Host "Ermittle Dateien zwischen $oldest und $newest"

        if ( ($path = (Get-ipxFoldername -rootpath g:\prj -description "Prüfordner angeben")) -ne $null) {
            if ( ( $csv = (Set-ipxFilename -rootpath \\verleihnix\Daten\Anwendungen\WindowsPowerShell\logs -title "CSV Datei abspeichern")) -ne $null) {
                $arr = @()
                Get-ChildItem -path $path -recurse -File |
                    Where-Object { ($_.LastWriteTime -ge $oldest -and $_.LastWriteTime -le $newest) -or ($_.CreationTime -ge $oldest -and $_.CreationTime -le $newest)} |
                     % {
                    $obj = New-Object PSObject
                    $obj | Add-Member NoteProperty Directory $_.DirectoryName
                    $obj | Add-Member NoteProperty Name $_.Name
                    $obj | Add-Member NoteProperty Extension $_.Extension
                    $obj | Add-Member NoteProperty Owner ((Get-ACL $_.FullName).Owner)
                    $obj | Add-Member NoteProperty LengthMB (Convert-ipxToMB $_.Length)
                    $obj | Add-Member NoteProperty CreationTime $_.CreationTime
                    $obj | Add-Member NoteProperty LastWriteTime $_.LastWriteTime
    
                $arr += $obj
                Write-Progress -Activity $_.Directory
                }
                $arr | Export-ipxCSV -Path $csv
            }
            else {
                Write-Host "Keine CSV-Datei angegeben." -ForegroundColor Yellow
            break;
            }
        }
        else {
            Write-Host "Kein Prüfordner angegeben" -ForegroundColor Yellow
        }
    }
}

#region Backup Filesystem
function New-ipxBackupFolder {
    param (
        [Parameter(Mandatory=$true,HelpMessage='Pfad zum Ziel')]
        [string]$Path,
        [Parameter(HelpMessage='Uhrzeit bei diff./ink. Backup anfügen')]
        [switch]$time,
        [Parameter(HelpMessage='Wochentag bei diff./ink. Backup anfügen')]
        [switch]$Weekday
    )

    if ( Test-Path -Path $Path ) {
        if ( $Weekday ) { $subfolder = [System.DateTime]::Today.ToString("dddd") }
        if ( $time ) { $subfolder = (Get-Date -Format yyMMdd) + "_" + (Get-Date -Format HHmm) }
        if ( !$Weekday -and !$time) { $subfolder = (Get-Date -Format yyMMdd) }
        if ( !(Test-Path -Path $Path\$subfolder)) {
            New-Item -Path $Path\$subfolder -ItemType Directory | Out-Null
        }
    }
    $Path + "\" + $subfolder
}

function Backup-ipxItem {
    <#
        .Synopsis
            Kopiert mittels Robocopy einen Ordner für Backupzwecke
        .Description
            Stellt eine Powershell Implementierung für Robocopy dar.
            Unterstützt werden Volles Backup (mit/ohne Remove-Archivebit), Differentiell und Inkrementelle Methode.
            Zusätzliche Optionen sind Dateifilter, Ordnerberechtigungen, Wiederholte Versuche und Wartezeit.
            Der Logfile backupitem_YYYYV.txt wird entweder im Zielverzeichnis oder im angegebenen Ordner angelegt.
            Nur bei mirror werden alle Informationen in den Logfile geschrieben.
            Der Zielordner muss existieren, sonst bricht robocopy ab.
        .Parameter Source
            Quellordner mit den zu kopierenden Dateien
        .Parameter Destination
            Zielordner wohin kopiert werden soll
        .Parameter Filter
            Dateifilter um nur bestimmte Dateitypen zu kopieren
        .Parameter Logpath
            Ordner in dem der Logfile angelegt werden soll, Vorgabe ist Destination
        .Parameter Mirror
            Spiegelt die Quelle im Ziel, löscht demnach auch nicht mehr vorhandene Dateien
        .Parameter Retry
            Wiederholungsversuche bei fehlerhaften Zugriffen, Vorgabe ist Null Sekunden
        .Parameter Wait
            Wartezeit bis zum nächsten Kopierversuch für eine Datei
        .Parameter ACL
            Kopiert die Sicherheitsinformationen und schreibt sie explizit ohne Vererbung auf das Ziel
        .Parameter time
            Fügt bei der differentiellen/inkrementellen Sicherung die Uhrzeit _HHmm an den Sicherungsordner
        .Example
            Backup-ipxItem -source C:\users\user\Downloads -destination d:
            Kopiert den Downloadordner des Benutzers nach D: und legt den Logfile ebenfalls unter D: ab.
        .Example
            Backup-ipxItem -source C:\users\user\Downloads -destination d: -logpath C:\users\user\documents
            Kopiert  ebenfalls den Downloadordner nach D, der Logfile wird aber im benutzerverzeichnis abgelegt.
        .Example
            Backup-ipxItem -source C:\users\user\Downloads -destination d: -removearchivebit
            Ebenfalls Kopie von Downloads nach D, es wird aber das Archivebit zurückgesetzt.
        .Example
            Backup-ipxItem -source C:\users\user\Downloads -destination d: -incremental
            Kopiert nur die geänderten Dateien in Downloads nach D:\yyMMdd\ (=aktuelles Datum) und setzt das Archivebit zurück.
        .Example
            Backup-ipxItem -source C:\users\user\Downloads -destination d: -differential
            Wie -incremental, jedoch wird das Archivbit nicht zurückgesetzt.
        .Example
            Backup-ipxItem -source C:\users\user\Downloads -destination d: -differential -time
            Zusätzlich wird durch -time die Uhrzeit eingefügt, also D:\yyMMdd_HHmm\
            Damit lässt sich mehrmals täglich ein Backup der geänderten Dateien erzeugen.
        .Example
            Backup-ipxItem -source C:\users\user\Downloads -destination d: -ACL
            Kopiert zusätzlich die (vererbten) Dateiberechtigungen (R/W ...) in explizite Rechte der Zieldateien.
        .Notes
            Version: 1.0.1.0
            Stand 23.2.2015
        .Inputs
            None
        .Outputs
            System.IO.Directory und System.IO.File
            gesicherte Dateien und logfile *.txt
    #>
    param (
        [Parameter(Mandatory=$true,HelpMessage='Pfad zur Quelle')]
        [string]$Source,
        [Parameter(Mandatory=$true,HelpMessage='Pfad zum Ziel')]
        [string]$Destination,
        [Parameter(HelpMessage='Dateifilter')]
        $filter = '',
        [Parameter(HelpMessage='Ordner für Logfile')]
        [string]$logpath = '',
        [Parameter(HelpMessage='Spiegelmodus, gelöschte werden entfernt')]
        [switch]$mirror,
        [Parameter(HelpMessage='X Wiederholungen bei Fehler')]
        [int]$retry = 0,
        [Parameter(HelpMessage='X Sekunden Wartezeit bei bevor erneut versucht wird zu kopieren')]
        [int]$wait = 10,
        [Parameter(HelpMessage='Zugriffsrechte kopieren und explizit auf Dateien anwenden')]
        [switch]$ACL,
        [Parameter(HelpMessage='Archivebit zurücksetzen')]
        [switch]$RemoveArchivebit,
        [Parameter(HelpMessage='Inkrementelles Backup in Ordner mit yyMMddhh')]
        [switch]$Incremental,
        [Parameter(HelpMessage='Differenzielles Backup in Ordner mit yyMMddhh')]
        [switch]$Differential
    )
    
    [string]$mir = ''
    [string]$rights = ''
    [string]$date= get-date -uformat %Y%V
    [string]$option1 = '/e'
    [string]$option2 = ''
    $attribute = [System.IO.FileAttributes]::archive
    
    if ($logpath -eq '' ) { $logpath = $destination }
    
    if ( $filter -eq '' ) { $filter = '*.*' }
    
    if ( $ACL ) { $rights = '/copy:DATSO' }


    if ( $mirror ) { 
        $option1 = '/e'
        $option2 = '/mir'
    }
    if ( $Incremental ) { 
        $option1 = '/s'
        $option2 = '/m'
    }
    if ( $Differential ) {
        $option1 = '/s'
        $option2 = '/a'
    }

    robocopy "$Source" "$Destination" $filter $option1 $option2 $rights /R:$retry /W:$wait /log+:$logpath\backupitem_$date.txt /ndl /nfl /v

    if ( $RemoveArchivebit) {
        Get-ChildItem -Path $Source -Recurse -File | Remove-ipxArchivebit
        }

}

function Backup-ipxItemToZip {
    <#
        .Synopsis
            Erstellt ein Zip-Archiv und fügt das aktuelle Datum an.
        .Description
            Die Funktion realisiert ein Backup als Zip-Archiv mit den Optionen Vollsicherung, inkrementell oder differentiell.
            Inkrementelles und differentielles Backup funktionieren über das Archivbit der Datei.
            Beim Vollbackup kann optional das Archivebit zurückgesetzt werden.
            Dateien mit einer Pfadlänge über 260 Zeichen werden nicht unterstützt und fehlen im Backup!
            Werden weitere Backupprogramme benutzt, muss der Umgang mit dem Archivebit einheitlich sein!
            Es wird zwingend die Powershell Community Extension (PSCX) benötigt !!!
        .Parameter Source
            Quellordner mit den zu kopierenden Dateien, kann auch eine Datei sein.
        .Parameter Destination
            Zielordner wohin das Archiv geschrieben wird, ohne abschließenden \.
        .Parameter Level
            Kompressionsgrad zwischen 1 und 9, Vorgabewert ist 4.
        .Parameter Logfile
            Gibt an ob ein Logfile am Zielspeicherort geschrieben werden soll.
        .Parameter Incremental
            Inkrementelle Sicherung, das Archivbit wird entfernt.
        .Parameter Differential
            Differentielle Sicherung, das Archivbit wird erhalten.
        .Parameter RemoveArchivebit
            Bei der Vollsicherung kann das Archivebit optional entfernt werden.
        .Parameter Time
            Fügt beim Archive zusätzlich die Uhrzeit "_HHmm" an.
        .Example
            Backup-ItemToZip -source C:\users\user\Downloads -destination d: -level 9
            Verpackt den Downloadordner des Benutzers auf dem Laufwerk D:\Downloads <Datum>.zip mit der höchsten Kompression.
        .Example
            Backup-ItemToZip -source C:\users\user\Downloads -destination d: -logfile
            Verpackt den Downloadordner des Benutzers auf dem Laufwerk D:\Downloads <Datum>.zip.
            Zusätzlich wird ein gleichnamiger Logfile D:\Downloads.txt erstellt der mittels -append fortgeführt wird.
            Im Logfile wird als Typ "Vollsicherung" vermerkt.
        .Example
            Backup-ItemToZip -source C:\users\user\Downloads -destination d: -logfile -RemoveArchivebit
            Verpackt den Downloadordner des Benutzers auf dem Laufwerk D:\Downloads <Datum>.zip.
            Das Archivebit wird entfernt.
            Im Logfile wird als Typ "Vollsicherung - Archivbit entfernt" vermerkt.
        .Example
            Backup-ItemToZip -source C:\users\user\Downloads -destination d: -logfile -Inremental
            Verpackt den Downloadordner des Benutzers auf dem Laufwerk D:\Downloads <Datum>.zip.
            Es werden nur geänderte Dateien mit Archivebit kopiert und deren Archivbit entfernt.
            Im Logfile wird als Typ "Inkrementelle Sicherung" vermerkt.
        .Example
            Backup-ItemToZip -source C:\users\user\Downloads -destination d: -logfile -Differential
            Verpackt den Downloadordner des Benutzers auf dem Laufwerk D:\Downloads <Datum>.zip.
            Es werden nur geänderte Dateien mit Archivebit kopiert und deren Archivbit erhalten.
            Im Logfile wird als Typ "Differentielle Sicherung" vermerkt.
        .Inputs
            Many
        .Outputs
            Zip-Archiv
    #>
    param (
        [Parameter(Mandatory=$true,HelpMessage='Pfad zur Quelle')]
        [string]$Source,
        [Parameter(Mandatory=$true,HelpMessage='Pfad zum Zielordner')]
        [string]$Destination,
        [Parameter(Mandatory=$false,HelpMessage='Kompressionsgrad, Vorgabe = 4')]
        [string]$Level = 4,
        [Parameter(Mandatory=$false,HelpMessage='Switch: Logfile schreiben')]
        [switch]$Logfile,
        [Parameter(Mandatory=$false,HelpMessage='Switch: Inkrementelle Sicherung, das Archivbit wird entfernt')]
        [switch]$Incremental,
        [Parameter(Mandatory=$false,HelpMessage='Switch: Differentielle Sicherung, das Archivbit wird erhalten')]
        [switch]$Differential,
        [Parameter(Mandatory=$false,HelpMessage='Switch: Archivbit entfernen, optional bei Vollbackup')]
        [switch]$RemoveArchivebit,
        [Parameter(Mandatory=$false,HelpMessage='Uhrzeit bei diff./ink. Backup anfügen')]
        [switch]$time
    )

        #terminierende PathTooLongException unterdrücken
        $BackupError = $ErrorActionPreference
        $ErrorActionPreference = "SilentlyContinue"

        if (!(Test-Pscx) ) {
            Write-Host "PSCX nicht verfügbar, Backup-ItemToZip kann nicht genutzt werden" -ForegroundColor Red
            break;
        }

        #Startzeit für Logfile
        if ($Logfile) {[string]$StartTime = 'Startzeit: {0}' -f (Get-Date) }
        
        #Quellpfad zerlegen für Dateinamen Archiv und Logfile und aktuelles Datum
        $path = ($Source).Split("\")
        [string]$file = $path[-1]
        $date= Get-Date -Format yyMMdd
        if ($time) { $date += "_" + (Get-Date -Format HHmm) }
        $attribute = [System.IO.FileAttributes]::archive

        if ( !$Incremental -and !$Differential ) {
            Get-ChildItem -Path $Source -Recurse -Force -OutVariable filelist | 
                Write-Zip -OutputPath "$Destination\$file $date.zip" -IncludeEmptyDirectories -EntryPathRoot $Source -Level $Level
            [string]$Backuptype = "Vollsicherung"
            if ( $RemoveArchivebit ) { $Backuptype += " - Archivbit entfernt" }
        }
        else {
            Get-ChildItem -Path $Source -Recurse -Force | 
                Where-Object { $_.Attributes -band $attribute } -OutVariable filelist | 
                Write-Zip -OutputPath "$Destination\$file $date.zip" -IncludeEmptyDirectories -EntryPathRoot $Source -Level $Level
            if ( $Incremental ) { 
                $RemoveArchivebit=$true
                [string]$Backuptype = "Inkrementelle Sicherung"
            }
            else { $Backuptype = "Differentielle Sicherung" }
            
        }
        
        if ( $RemoveArchivebit) {
            foreach ($element in $filelist.FullName ) { $element | Remove-Archivebit }
        }        

        if ($Logfile) {
            [string]$zipArchive = "$Destination\$file $date.zip"
            $fileSize = (Get-Item -Path "$Destination\$file $date.zip").Length | foreach { Convert-ToMB($_) }
            $fileSize = $fileSize.ToString()
            $fileSize = "Dateigröße: " + $fileSize + " MB"
            [string]$endTime = 'Startzeit: {0}' -f (Get-Date)
            
            "--- Backup-ItemToZip ---" | Out-File -FilePath "$Destination\$file.txt" -Append
            $Backuptype | Out-File -FilePath "$Destination\$file.txt" -Append 
            $zipArchive | Out-File -FilePath "$Destination\$file.txt" -Append
            $fileSize | Out-File -FilePath "$Destination\$file.txt" -Append
            $startTime | Out-File -FilePath "$Destination\$file.txt" -Append
            $endTime | Out-File -FilePath "$Destination\$file.txt" -Append
            "" | Out-File -FilePath "$Destination\$file.txt" -Append
        }
    $ErrorActionPreference = $BackupError
    
}
#endregion

function Compress-ipxArchive {
    <#
        .Synopsis
            Komprimiert das angegebene Element als 7z-Archiv.
        .Description
            Verwendet zur Komprimierung die mitgelieferte 7za.exe.
            Die Zieldatei muss vollqualifiziert mit Dateiendung .7z sein.
        .Parameter Source
            Die Datei oder der Ordner zum Packen
        .Parameter Destination
            Der Ordnerpfad an dem das Archiv abgelegt wird.
        .Example 
            Compress-ipxArchive -Source D:\MeinOrdner -Destination D:\GepackterOrdner.7z
            Packt den Ordner MeinOrdner im Laufwerk D als GepackterOrdner.7z Archiv.
        .Inputs
            Source System.IO.File oder System.IO.Folder
            Destination System.IO.File 7z-Archiv
        .Outputs
            System.IO.File *.7z
        .Notes
            Version: 1.0.0.0
            Datum: 9.1.2017
        .Component
            exe\7za.exe
    #>

    param (
    [Parameter(Mandatory,HelpMessage='Pfad zur Quelle')]
    [string]$Source,
    [Parameter(Mandatory,HelpMessage='Zieldatei')]
    [string]$Destination
    )
    $zipexe = Get-ipx7zip
    &$zipexe a -t7z -bt $Destination $Source
}

function Expand-ipxArchive {
    <#
        .Synopsis
            Entpackt ein 7z-Archiv in den angegebenen Ordner.
        .Description
            Verwendet zum Entpacken die mitgelieferte 7za.exe.
            Im Zielordner wird die Datenstruktur wie im Archiv erstellt.
            Der FLat-Parameter erlaubt das entpacken aller Ordner und Dateien direkt in den Zielordner.
        .Parameter Source
            Vollqualifierter Name zum 7z-Archiv.
        .Parameter Destination
            Gibt den Ordner an, in den das Archiv entpackt wird.
        .Parameter Flat
            7Zip entpackt das Archiv flach in den Zielordner.
        .Example 
            Expand-ipxArchive -Source D:\GepackterOrdner.7z -Destination D:\EntpackterOrdner
            Das Archiv wird in den Ordner Entpackter Ordner mit seiner Struktur entpackt.
        .Example
            Expand-ipxArchive -Source D:\GepackterOrdner.7z -Destination D:\EntpackterOrdner -Flat
            Das Archiv wird in den Ordner Entpackter Ordner entpackt, dabei werden alle
            Dateien und Ordner direkt in \EntpackterOrdner abgelegt.
        .Inputs
            Source System.IO.File 7z-Archiv
            Destination System.IO.Folder
            Optional Switch Flat
        .Outputs
            System.IO.Files und System.IO.Folder
        .Notes
            Version: 1.0.0.0
            Datum: 9.1.2017
        .Component
            exe\7za.exe
    #>

    param (
    [Parameter(Mandatory,HelpMessage='Pfad zum 7z-Archiv')]
    [System.IO.FileInfo]$Source,
    [Parameter(Mandatory,HelpMessage='Zielordner')]
    [string]$Destination,
    [Parameter(HelpMessage='Alles im Zielordner entpacken')]
    [switch]$Flat
    )
    $zipexe = Get-ipx7zip
    $Destination = "-o" + $Destination
    if ( $flat ) { &$zipexe e $Source $Destination }
    else { &$zipexe x $Source $Destination }
}

function Get-ipxFolderlist {
    <#
        .Synopsis
            Ermittelt alle Unterordner in einem Pfad bis zu einer angegebenen Tiefe.
        .Description
            Die Funktion ermittelt anhand des angegebenen Pfades alle Ordner.
            Die Ordnertiefe kann über den Parameter $depth vorgegeben werden.
        .Parameter path
            Gibt den Ordner an, in dem nach allen Unterordnern gesucht werden soll.
        .Parameter depth
            Gibt die zu ermittelnde Tiefe an. Vorgabe ist 1, also nur die direkten Unterordner von $path.
        .Example 
            Get-Folderlist -path C.\users\user -depth 2
            Ermittelt im Benutzerverzeichnis von user die Unterordner bis zur zweiten Ebene.
        .Example
            Get-Folderlist -path C.\users\user -depth 2 | Select-Object -property FullName
            Ermittelt im Benutzerverzeichnis von user die Unterordner bis zur zweiten Ebene.
            Filtert zusätzlich die Ausgabeobjekte und gibt nur den vollen Ordnerpfad zurück.
        .Inputs
            System.String Dateipfad und System.Int16 Ordnertiefe
            Dateipfad kann Werte von Pipeline empfangen
        .Outputs
            System.IO.Directory in System.Array
        .Notes
            Version: 1.0.0.0
            Datum: 23.2.2015
        .Component
            Standard Powershell
    #>
    
    param (
        [Parameter(Mandatory,HelpMessage='Grundpfad der Projekte')]
        [Alias('Fullname')]
        [string]$path,
        [Parameter(HelpMessage='Ordnertiefe angeben, Vorgabe ist 1')]
        [int]$depth=1
    )

    #PS4 auf 2012R2 mit Exchange 2016 CU5 kann das nicht (25.5.2017)
    #$Folderlist = [System.IO.DirectoryInfo]::new($path)
    $Folderlist = Get-Item -Path $path

    For ($i=0; $i -lt $depth; $i++) {
        Write-Host 'Ermittle Ordner in Ebene '$i
        foreach($Folder in $Folderlist) {
            $Folders = Get-ChildItem -Path $Folderlist.FullName -Directory -ErrorAction Ignore
        }
        Write-Host 'Ermittelte Ordner in Ebene '$i':' $Folders.Length
        $Folderlist = $Folders
    }
    $Folderlist

}

function Get-ipxFoldersize {
<#
    .Synopsis
        listet die Ordnergröße in der angegebenen Ordnertiefe und gibt sie als Gridview oder csv aus
    .Description
        Die Funktion fragt einen Rootordner sowie eine Ordnertiefe nach und ermittelt die Ordnergrößen dieser Hierarchietiefe.
        Der Schalter -gb ermittelt die Gigybate, zusätzlich zur Standardausgabe in Mgeabyte.
        Die Funktion erstellt ein Array mit Spalten für FullName des Ordners, die Größe in MB und optional GB.
        logfile verlangt einen vollen Namen und das Dateiformat .csv, showwindow gibt das Ergebnis als Gridview aus.
    .Parameter path
        Voll qualifizierter Name oder Powershell-Drives wie $home
    .Parameter depth
        optionale Ordnertiefe, Vorgabe ist 0. Bei 0 wird nur path durchsucht, bei 1 entsprechend eine Hierarchietiefe darunter und path selbst.
    .Parameter logfile
        Angegeben werden muss der vollständige Pfad mit Datei wie $home\documents\logfile.csv
        Wenn ein logfile angegeben wird ist keine Gridview möglich!
    .Parameter showwindow
        Gibt die ermittelten Werte als Gridview aus.
        Wenn Gridview gewählt wird ist kein logfile möglich!
    .Parameter gb
        Ermittelt zusätzlich die Ordnergröße in Gigabyte.
    .Parameter filter
        Dateifilter wie *.jpg der die Suche auf diesen Dateityp eingrenzt.
    .Notes
        Version: 2.1.0.0
        Datum: 23.2.2015
    .Link
        http://www.imaginado.de
    .Example
        Get-ipxFoldersize -path $home -showwindow
        Ermittelt die Größe des Benutzerverzeichnisses in Megabyte und gibt es als Gridview aus.
    .Example
        Get-ipxFoldersize -path $home -depth 1 -showwindow
        Ermittelt im Benutzerverzeichnis die Ordnergrößen für Dokumente, Eigene Bilder, Eigene Musik, ... und gibt es als Gridview aus.
    .Example
        Get-pifFoldersize -path $home -depth 1 -showwindow -filter *.jpg
        Ermittelt im Benutzerverzeichnis die Ordnergrößen Bilddateien *,jpg und gibt es als Gridview aus, auch "leere" Ordner.
    .Example
        Get-ipxFoldersize -path \\server\projects -depth 2 -gb -logfile $home\documents\projectlist.csv
        Ermittelt in einer Serverfreigabe die Ordner der zweiten Hierarchietiefe.
        Dies bietet sich an wenn beispielsweise Projekte folgendermaßen strukturiert sind: \\server\projects\1000\1001-projectname
        Das Skript ermittelt bei der angegebenen Tiefe 2 die Größe der eigentlichen Projektordner "1001-projectname".
        Der Schalter -gb ermittelt zusätzlich die Größe in Gigabyte, bei Projekten oder Multimediaarchiven ist dies sinnvoller.
        Zum Schluss wird das Ergebnis im Benutzerverzeichnis unter Dokumente als csv abgespeichert.
    .Inputs
        Many
    .Outputs
        Gridview oder System.IO.File logfile *.csv
#>
    param (
    [Parameter(Mandatory, HelpMessage='Grundpfad der Projekte')]
    $path,
    [Parameter(HelpMessage='Ordnertiefe angeben')]
    $depth=0,
    [Parameter(ParameterSetName='log',HelpMessage='Pfad zur Ausgabedatei/Logfile mit Endung csv')]
    [String]$logfile,
    [Parameter(ParameterSetName='Grid')]
    [switch]$ShowWindow,
    [Parameter(HelpMessage='Mit Ordnergröße in Gigabyte ausgeben')]
    [Switch]$gb,
    [Parameter(HelpMessage='Bei depth > 0 den rootpath mit ausgeben')]
    [Switch]$root,
    [Parameter(HelpMessage='Filter um nur nach bestimmten Dateitypen zu suchen')]
    [string]$filter= '*'
    )  

    'Startzeit: {0}' -f (Get-Date)

    $projectfolders = Get-ipxFolderlist -path $path -depth $depth

    $percentage = 0
    $folderarray = @()

    foreach($project in $projectfolders.FullName){
        if (Test-Path -Path $project) {
            Write-Progress -Activity $project -PercentComplete ($percentage/$projectfolders.Length*100)
            $foldersize = Get-ipxSize -path $project -filter $filter
            $obj = New-Object PSObject
            $obj | Add-Member NoteProperty FullName $project
            $obj | Add-Member NoteProperty 'Foldersize in MB' (Convert-ipxToMB($foldersize))
            if ( $gb) { $obj | Add-Member NoteProperty 'Foldersize in GB' (Convert-ipxToGB($foldersize)) }

            $folderarray += $obj
            $percentage ++
        }
    }

    if ( $root ) {
        if (Test-Path -Path $path) {
            Write-Progress -Activity "Ermittle root"
            $foldersize = Get-ipxSize -path $path -root -filter $filter
            $obj = New-Object PSObject
            $obj | Add-Member NoteProperty FullName $path
            $obj | Add-Member NoteProperty 'Foldersize in MB' (Convert-ipxToMB($foldersize))
            if ( $gb) { $obj | Add-Member NoteProperty 'Foldersize in GB' (Convert-ipxToGB($foldersize)) }

            $folderarray += $obj
        }
    }

    If ($ShowWindow){$folderarray | Out-GridView -Title "Get-Foldersize: $path | Tiefe: $depth | Filter: $filter"}
    Else {$folderarray | Export-ipxCSV -Path "$logfile"}

    'Endzeit: {0}' -f (Get-Date)

}

function Get-ipxSize {
<#
    .Synopsis
        Ermittelt die Ordnergröße des angegebenen Ordners.
    .Description
        Die Ordnergröße wird in Byte ermittelt, lässt sich aber über den Positionsparameter -f in KB, MB oder GB wandeln - siehe Beispiel über Get-Help.
        Get-Foldersize wird für die Ordnergrößenermittlung in Get-Projectsize verwendet!
    .Parameter path
        Pfadangabe, vollqualifiziert wie D:\ordner oder Powershell Drive wie $home
    .Parameter Filter
        Dateifilter um zB alle JPEG eines Ordners zu ermitteln
    .Notes
        Version 1.1.0.0
        Datum 23.2.2015
        Angepasste Funktion auf Grundlage von "Scripting mit Windows Powershell 3.0 - Der Workshop (MS Press)"
    .Example
        '{0:0.00}' -f ((Get-ipxFoldersize -path $HOME)/1MB)
        Ermittelt die Odnergröße des Benutzerverzeichnisses und gibt sie in Megabyte mit zwei Kommastellen aus.
    .Inputs
        System.IO.DirectoryInfo path als String
        System.String filter as String
    .Outputs
        System.Double
    .Component
        Standard Powershell
#>
    param (
        [Parameter(Mandatory, HelpMessage='Pfad zum Ermitteln der Ordnergröße')]
        [string]$path,
        [Parameter(HelpMessage='Ermittelt nur die Dateien im Stammordner')]
        [switch]$root,
        [Parameter(HelpMessage='Filter um nur nach bestimmten Dateitypen zu suchen')]
        [string]$filter= '*'
    )
    [double]$foldersize = 0
        if ( !$root) {
            $foldersize = (Get-ChildItem -Path $path -Filter $filter -Recurse -ErrorAction SilentlyContinue -Force |
            Measure-Object -Property Length -Sum -ErrorAction Ignore |
            Select-Object -ExpandProperty Sum)
        }
        else {
            $foldersize = (Get-ChildItem -Path $path -Filter $filter -ErrorAction SilentlyContinue -Force |
            Where-Object {$_.GetType() -eq [System.IO.FileInfo] } |
            Measure-Object -Property Length -Sum -ErrorAction Ignore |
            Select-Object -ExpandProperty Sum)
        }
    $foldersize
}

function Test-ipxPathlengthExtra {
<#
    .Synopsis
        Testet die Pfadlänge des vollständigen Dateinamens und gibt sie als aus.
    .Description
        Die Maximallänge ist auf 248 Zeichen voreingestellt,
        das Skript kann jedoch prinzipbedingt nicht mit mehr als 260 Zeichen umgehen.
        Das Ergebnis kann entweder als Gridview oder als CSV ausgegeben werden.
    .Parameter pathlegnth
        Gibt die Pfadlänge an, Vorgabe ist 248
    .Parameter path
        Gibt den zu prüfenden Pfad an.
    .Parameter logpath
        Gibt die csv-Datei für den Logfile an
    .Parameter ShowWindow
        Statt dem logfile wird das Ergebnis als Gridview ausgegeben
    .Notes
        Version: 1.1.0.0
        Datum: 23.2.2015
    .Inputs
        System.Int16 pathlength Pfadlänge
        System.IO.DirectoryInfo path als String
        System.IO.DirectoryInfo logpath als String
    .Outputs
        Gridview oder System.IO.File logfile *.csv
    .Component
        None
#>
    param(
        [Parameter(HelpMessage='Maximale Pfadlänge, Vorgabe ist 248')]
        [Int16]$pathlength=248,
        [Parameter(Mandatory,HelpMessage='Pfad zum Unterordner')]
        $path,
        [Parameter(HelpMessage='Pfad zur Ausgabedatei/Logfile mit Endung csv')]
        $logpath,
        [switch]$ShowWindow
    )

    $array = @()
    
    Get-ChildItem $path -recurse | % {
        $a = $_.Fullname | Measure-Object -Character
        if ($a.Characters -gt $pathlength){
            $obj = New-Object PSObject
            $obj | Add-Member NoteProperty FullName $_.FullName
            $obj | Add-Member NoteProperty Directory $_.DirectoryName
            $obj | Add-Member NoteProperty Name $_.Name
            $obj | Add-Member NoteProperty BaseName $_.BaseName
            $obj | Add-Member NoteProperty Owner ((Get-Acl $_.FullName).Owner)
            $obj | Add-Member NoteProperty Length $_.Length
            $obj | Add-Member NoteProperty Pathlength $a.Characters

            $array += $obj
        }
    }

    If ($ShowWindow){$array | Out-GridView -Title "Dateien mit Pfadlänge über $pathlength Zeichen"}
    Else {$array | Export-ipxCSV -Path "$logpath"}
}

function Test-ipxPathLength {
<#
    .Synopsis
        Testet die Pfadlänge des vollständigen Dateinamens.
    .Description
        Die Maximallänge ist auf 248 Zeichen voreingestellt,
        das Skript benutzt robocopy und ermittelt auch Pfadlängen über 260 Zeichen.
        Es wird nur der FullName und die Pfadlänge ermittelt.
        Das Ergebnis kann entweder als Gridview oder als CSV ausgegeben werden.
    .Parameter pathlegnth
        Gibt die Pfadlänge an, Vorgabe ist 248
    .Parameter path
        Gibt den zu prüfenden Pfad an.
    .Parameter logpath
        Gibt die csv-Datei für den Logfile an
    .Parameter ShowWindow
        Statt dem logfile wird das Ergebnis als Gridview ausgegeben
    .Notes
        Version: 1.0.0.0
        Datum: 23.2.2015
    .Inputs
        System.Int16 pathlength Pfadlänge
        System.IO.DirectoryInfo path als String
        System.IO.DirectoryInfo logpath als String
    .Outputs
        Gridview oder System.IO.File logfile *.csv
    .Component
        Robocopy
#>
    param(
        [Parameter(Mandatory,HelpMessage='Pfad zum Unterordner')]
        $path,
        [Parameter(HelpMessage='Maximale Pfadlänge, Vorgabe ist 248')]
        $pathlength=248,
        [Parameter(HelpMessage='Pfad zur Ausgabedatei/Logfile mit Endung csv')]
        $logpath,
        [switch]$ShowWindow
    )

    $array = @()
    robocopy "$path" "D:\robocopy" /e /np /fp /njh /njs /r:0 /w:0 /l | ForEach-Object { if ($_.Length -gt $pathlength) {
        #$position = $null
        $position = $_.IndexOf("$path")
        $obj = New-Object PSObject
        $obj | Add-Member NoteProperty FullName ($_.SubString($position))
        $obj | Add-Member NoteProperty Length $_.Length
        $array += $obj
    }
    }
    If ($ShowWindow){$array | Out-GridView -Title "Dateien mit Pfadlänge über $pathlength Zeichen"}
    Else {$array | Export-ipxCSV "$logpath" }
}

function Get-ipxSharedDrive {
    <#
        .Synopsis
            WIP: Ermittelt die verbundenen Netzlaufwerke
        .Description
            Ermittelt beim lokalen oder RemoteComputer die verbundenen Netzlaufwerke
            und gibt den Laufwerksbuchstaben und UNC-Pfad aus
        .Inputs
            None
        .Outputs
            System.Management.Automation.PSCustomObject
        .Notes
            Version 1.0.0.1
            Datum: 23.2.2015
        Component
            Cim Instance
    #>

    Get-CimInstance Win32_MappedLogicalDisk | Select-Object -Property Caption,ProviderName
}

function Remove-ipxEmptyFolders {
    <#
        .Synopsis
            Entfernt leere Ordner in einem Verzeichnis
        .Description
            Es wird eine Schleife fünf Mal durchlaufen und immer der tiefste leere Ordner gelöscht.
            Sind die Strukturen noch tiefer kann die Funktion schlichtweg mehrfach ausgeführt werden.
        .Parameter interactive
            Anstatt den Pfadnamen per path zu übermitteln wird ein Dialogfeld benutzt
        .Parameter path
            Pfad zum Ordner der geprüft werden soll, nur ohne interactive nutzbar
        .Parameter cycle
            Gibt Anzahl an Wiederholungen und damit implizit die zu löschende Ordnertiefe an, Vorgabe ist 3
        .Example
            Remove-ipxEmptyFolders -path c:\users\user -cycle 1
            Durchsucht das Benutzerverzeichnis nach einzelnen leeren Ordnern
        .Notes
            Version: 1.0.0.1
            Datum 23.2.2015
        .Inputs
            System.SwitchParameter interactive oder System.IO.DirectoryInfo als System.String path
            System.Int16 cycle
        .Outputs
            None
        .Component
            None
    #>
    param (
        [Parameter(HelpMessage='Ordnerwahl mit Dialogfenster',ParameterSetName='Interactive')]
        [switch]$interactive,
        [Parameter(HelpMessage='direkte Pfadangabe ohne Dialog',ParameterSetName='console')]
        [string]$path='',
        [Parameter(HelpMessage='Wie oft soll duchlaufen werden')]
        [int]$cycle=3
    )
    if ( $interactive ) {
        $path = (Get-ipxFoldername -rootpath d: -description "Prüfordner angeben")
    }
    if ( $path -ne $null) {
        for($i = 1; $i -le $cycle; $i++){
            Write-Progress -Activity "Durchlauf $i/$cycle"
            Get-ChildItem -Path $path -Directory -Recurse |
            Where-Object {($_.GetDirectories().Count -eq 0) -and ($_.GetFiles().Count -eq 0)} |
            Remove-Item -Force
        }
    }
    else {
        Write-Host "Kein Ordner angegeben." -ForegroundColor Yellow
    }
}

function Get-ipxArchivebit {
    <#
    .Synopsis
        Ermittelt Dateien anhand des Archivebits.
    .Description
        Im angegebenen Ordner selbst oder rekursiv durch alles Unterordner
        werden die Dateien ermittelt die ein Archivebit gesetzt haben.
        Alternativ können über "inverse" als Dateien ohne Archivebit ermittelt werden.
        Über "filter" können Dateitypen wie *.jpg angegeben werden.
    .Parameter path
        Ordnerpfad in dem die Dateien ermittelt werden sollen.
    .Parameter filter
        Optionaler Dateifilter wie *.jpg oder *.avi.
    .Parameter recurse
        Aktiviert die rekursive Suche in allen Unterordnern von $path.
    .Parameter inverse
        Invertiert die Suche und findet alle Dateien ohne Archivebit.
    .Example
        Get-ipxArchivebit -path C:\users\user\downloads
        Ermittelt alle Dateien mit Archivebit im Downloadordner eines Benutzers "user".
    .Example
        Get-ipxArchivebit -path C:\users\user\downloads -recurse
        Ermittelt alle Dateien mit Archivebit im Downloadordner und den Unterordnern eines Benutzers "user".
    .Example
        Get-ipxArchivebit -path C:\users\user\downloads -inverse -recurse
        Ermittelt alle Dateien ohne Archivebit im Downloadordner und den Unterordnern eines Benutzers "user".
    .Example
        Get-ipxArchivebit -path C:\users\user\downloads -inverse -recurse | Set-ipxArchivebit
        Ermittelt rekursiv alle Dateien ohne Archivebit und setzt es danach über die Pipeline.
    .Example
        Get-ipxArchivebit -path C:\users\user\downloads -recurse | Remove-ipxArchivebit
        Ermittelt rekursiv alle Dateien mit Archivebit und entfernt es danach über die Pipeline.
    .Inputs
        System.IO.DirectoryInfo als System.String path
        System.String filter
        System.SwitchParameter recurse und inverse
    .Outputs
        System.Array
    .Notes
        Version: 1.0.0.0
        Datum: 1.3.2015
    .Component
        None
    #>

    param (
        [Parameter(Mandatory,HelpMessage='Ordnerpfad zu Dateien')]
        [string]$path,
        [Parameter(HelpMessage='Dateifilter')]
        [string]$filter='',
        [Parameter(HelpMessage='rekursiv oder nur angegebener Ordner')]
        [switch]$recurse,
        [Parameter(HelpMessage='Sucht Dateien ohne Archivbit')]
        [switch]$inverse
    )
    
    $attribute = [System.IO.FileAttributes]::archive
    if ( $recurse ) {
        Get-ChildItem -Path $path -Filter $filter -Recurse -File |
            ForEach-Object {
                if ( !$inverse ) {
                    if ( (Get-ItemProperty -Path $_.FullName).Attributes -band $attribute ) { $_ }
                }
                else { 
                    if ( (Get-ItemProperty -Path $_.FullName).Attributes -band $attribute ) { <#<do nothing#> }
                    else { $_ }
                }
            }
    }
    else {
        Get-ChildItem -Path $path -Filter $filter -File |
            ForEach-Object {
                if ( !$inverse ) {
                    if ( (Get-ItemProperty -Path $_.FullName).Attributes -band $attribute ) { $_ }
                }
                else { 
                    if ( (Get-ItemProperty -Path $_.FullName).Attributes -band $attribute ) { <#<do nothing#> }
                    else { $_ }
                }
            }
    }
}

function Set-ipxArchivebit {
    <#
    .Synopsis
        Setzt das Archivebit für Dateien die es nicht gesetzt haben.
    .Description
        In einer Pipeline werden erst Dateien ermittelt die der Funktion übergeben werden,
        die dann das Archivebit setzt, wenn es nicht vorhanden ist. 
    .Parameter path
        Voller Dateipfad der von der Pipeline kommt und als Alias 'FullName' gemapped wird.
    .Example 
        Get-ipxArchivebit -path C:\users\user\downloads -inverse -recurse | Set-ipxArchivebit
        Ermittelt rekursiv alle Dateien ohne Archivebit und setzt es danach über die Pipeline.
    .Inputs
        System.IO.DirectoryInfo als System.String path
    .Outputs
        None
    .Notes
        Version. 1.0.0.0
        Datum: 1.3.2015
    .Component
        None
    #>

    param (
        [Parameter(Mandatory,HelpMessage='Dateipfad aus der Pipeline',ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true)]
        [Alias('Fullname')]
        [string]$path
    )

    begin {
        $attribute = [System.IO.FileAttributes]::archive
    }

    process {
        if ((Get-ItemProperty -Path $path).Attributes -band $attribute ) { <#do nothing#> }
        else {
            Set-ItemProperty -Path $path -Name Attributes -Value ((Get-ItemProperty -Path $path ).Attributes -bxor $attribute) -ErrorAction SilentlyContinue
        }
    }
}

function Remove-ipxArchivebit {
    <#
    .Synopsis
        Entfernt das Archivebit für Dateien die es gesetzt haben.
    .Description
        In einer Pipeline werden erst Dateien ermittelt die der Funktion übergeben werden,
        die dann das Archivebit entfernt, wenn es vorhanden ist.
        Findet Verwendung bei Backup-Item und Backup-ItemToZip. 
    .Parameter path
        Voller Dateipfad der von der Pipeline kommt und als Alias 'FullName' gemapped wird.
    .Example 
        Get-ipxArchivebit -path C:\users\user\downloads -recurse | Remove-ipxArchivebit
        Ermittelt rekursiv alle Dateien mit Archivebit und entfernt es danach über die Pipeline.
    .Inputs
        System.IO.DirectoryInfo als System.String path
    .Outputs
        None
    .Notes
        Version. 1.0.0.0
        Datum: 1.3.2015
    .Component
        None
    #>

    param (
        [Parameter(Mandatory,HelpMessage='Dateipfad aus der Pipeline',ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true)]
        [Alias('Fullname')]
        [string]$path
    )

    begin {
        $attribute = [System.IO.FileAttributes]::archive
    }

    process {
        if ((Get-ItemProperty -Path $path).Attributes -band $attribute ) {
            Set-ItemProperty -Path $path -Name Attributes -Value ((Get-ItemProperty -Path $path ).Attributes -bxor $attribute)
        }
    
    }
}

function Get-ipxFolderFromSearchfile {
    <#
        .Synopsis
            Ermittelt anhand einer Textdatei mit Ordnernamen deren vollständigen Pfad im zu durchsuchenden Ordner
        .Description
            Zuerst wird die Datei mit den zu suchenden Ordnernamen angegeben, diese wird eingelesen und nach Übereinstimmungen
            im angegebenen Suchordner gesucht. Übereinstimmungen werden als Objekt mit den Eigenschaften ausgegeben.
        .Parameter interactive
            Aktiviert die interaktive Ordnerwahl mit Dialog
        .Parameter searchlist
            Angabe der Suchliste als txt in der Konsole
        .Parameter folderpath
            Der zu durchsuchende Ordner für die Analyse
        .Parameter depth
            Ordnertiefe bis zu der durchsucht werden soll
        .Example 
            Get-ipxFolderFromSearchfile -interactive -depth 3
            Interaktive Ordner- und Dateiwahl mit einer Tiefe von drei Ordnern
        .Inputs
            System.SwitchParameter interactive
            System.IO.FileInfo als System.String searchlist
            System.IO.DirectoryInfo als System.String folderpath
            System.Int16 depth
        .Outputs
            System.Array
        .Notes
            Version: 0.0.0.1
            Datum: 1.3.2015
            Cmdlet ist noch in der Entwicklung
        .Component
            Get-ipxFolderlist
    #>

    param (
        [Parameter(HelpMessage='Interaktive Ordnerwahl', ParameterSetName='Interactive')]
        [switch]$interactive,
        [Parameter(HelpMessage='TXT Datei mit den Suchbegriffen', ParameterSetName='Console')]
        [string]$searchlist,
        [Parameter(HelpMessage='Ordner mit den zu ermittelnden Unterordnern', ParameterSetName='Console')]
        [string]$folderpath,
        [Parameter(HelpMessage='Ordnertiefe in der gesucht werden soll')]
        [int]$depth
    )

    $result = @()
    
    if ( $interactive ) {
        $searchlist = Get-ipxFilename -title "Textdatei wählen" -filter "TXT|*.txt" -rootpath C:
        $folderpath = Get-ipxFoldername -description "Suchordner angeben"
    }

    $list = Get-Content -Path $searchlist
    $folders = Get-ipxFolderlist -path $folderpath -depth $depth


    foreach ( $element in $list ) {
        foreach ( $folder in $folders ) {
            if ( $folder.Name -like "*$element*" ) {
                $result += $folder
            }
        }
    }
    $result
}

function Get-ipxChildItemOfOwner {
    <#
        .Synopsis
            Ermittelt alle Dateien und Objekte eines Benutzers
        .Description
            Dem Cmdlet müssen ein Ordnerpfad und der Benutzer übergeben werden.
            Optional lassen sich Filter für Datei- und Ordnernamen, nur Dateien oder nur Ordner angeben.
            Die rekursive Suche kann ebenfalls optional aktiviert werden.
            Danach werden alle Objekte ermittelt die im Besitz des angegebenen Benutzers sind.
        .Parameter Path
            Dateipfad zum Ordner der durchsucht wird.
        .Parameter Filter
            Filter wie *.doc* um alle Word Dokumente zu filtern.
        .Parameter Recurse
            Switch Parameter der die rekursive Suche auslöst.
        .Parameter All
            Switch Parameter der nur für DefaultParameterSetName benutzt wird und versteckt ist - funktionslos.
        .Parameter File
            Switch Parameter der nur Dateien ausgibt.
        .Parameter Directory
            Switch Parameter der nur Ordner ausgibt.
        .Parameter Owner
            Gibt den Benutzer wie "Mueller" ohne Domäne an.
        .Example 
            Get-ipxChildItemOfOwner -Path \\server\share -Recurse -Owner mueller
            Ermittelt in einer Dateifreigabe alle Dateien und Ordner des Benutzers mueller rekursiv.
        .Example 
            Get-ipxChildItemOfOwner -Path \\server\share -Recurse -Directory -Owner mueller
            Ermittelt alle Ordner in einer Dateifreigabe des Benutzers mueller rekursiv.
        .Example 
            Get-ipxChildItemOfOwner -Path \\server\share -Recurse -File -Filter *.doc* -Owner mueller
            Ermittelt alle Excel Dokumente in einer Dateifreigabe des Benutzers mueller rekursiv.
        .Inputs
            String Path als System.IO.Directory
            String Filter
            Switch Recurse
            Switch File
            Switch Directory
            String Owner
        .Outputs
            System.IO.Directory und System.IO.File
        .Notes
            Version 1.0.0.0
            Datum 8.5.2015
            Parameter All, File und Directory sind sich gegenseitig ausschließende ParameterSets, All ist das Standardverhalten.
        .Component
            Standard Powershell
    #>
    [CmdLetBinding(DefaultParameterSetName='All')]

    param (
        
        [Parameter(Mandatory,HelpMessage='Pfad zur Quelle')]
        [string]$Path,
        [Parameter(HelpMessage='Namensfilter')]
        [string]$Filter='*',
        [Parameter(HelpMessage='Rekursive Suche')]
        [switch]$Recurse,
        [Parameter(DontShow,HelpMessage='Datein und Ordner - Standardverhalten',ParameterSetName='All')]
        [switch]$All,
        [Parameter(HelpMessage='Nur Dateien ausgeben',ParameterSetName='File')]
        [switch]$File,
        [Parameter(HelpMessage='Nur Ordner ausgeben',ParameterSetName='Directory')]
        [switch]$Directory,
        [Parameter(Mandatory,HelpMessage='Besitzer')]
        [string]$Owner
    )

    $Parameter = @{
        Path = $Path;
        Filter = $Filter
    }
    if ( $Recurse ) { $Parameter.Recurse = $true }
    if ( $File ) { $Parameter.File = $true }
    if ( $Directory ) { $Parameter.Directory = $true }
    
    Get-ChildItem @Parameter | Where-Object { ((Get-ACL $_.FullName).Owner).Split('\')[-1] -eq "$Owner" }
}

function Get-ipxItemList {
    <#
        .Synopsis
            Dateisuche mit einer simplen Benutzeroberfläche.
        .Description
            Ruft ein WPF XAML Anwendungsfenster auf, in dem ein Ordner für die Dateisuche gewählt wird.
            Als Optionen sind die rekursive Suche und ein einfacher Dateifilter integriert.
            Per Rechtsklick auf ein gefundenes Element kann es selbst oder sein Ordner im Explorer geöffnet werden.
            Die Routine funktioniert auch auf Netzlaufwerken, bei denen WIndows selbst häufig sehr langsam sucht und/oder nichts findet.
            Das Cmdlet stellt somit die GUI-Variante von Get-Childitem dar die Endanwender benutzen können.
        .Inputs
            None
        .Outputs
            None
        .Notes
            Version: 1.1.0.0
            Datum: 1.12.2016
        .Component
            xaml\GetItemlist
    #>

    Import-ipxXAML -Path $MyModuls\Modules\ipx\xaml\GetItemList.xaml

    #EVENT Handler
    $PathButton.add_Click({
        $PathTextBox.Text = Get-ipxFoldername -rootpath C: -description "Suchordner angeben"
    })

    $InvokeElement.add_Click({
        $selectedRow = $ItemGrid.SelectedValue
        Invoke-Item -LiteralPath $selectedRow.FullName
    })
    
    $InvokePath.add_Click({
        $selectedRow = $ItemGrid.SelectedValue
        Explorer.exe $selectedRow.Directory
    })

    $StartButton.add_Click({
        $ItemGrid.Items.Clear()
        $Parameterlist = @{
            Path = $PathTextBox.Text.ToString()
            Filter = $FilterTextBox.Text.ToString()
            Recurse = $RecurseSearch.IsChecked
        }
        Get-ChildItem @Parameterlist -File | Add-Member -MemberType ScriptProperty -Name LengthMB -Value {Convert-ipxToMB($this.Length)} -PassThru | Select-Object -Property Fullname,Directory,Name,LengthMB,LastWriteTime | ForEach-Object {
            $ActualElement = $_.FullName
            $xamGUI.Dispatcher.Invoke( "Render", ([Action``2[[System.Object,mscorlib],[System.Object, mscorlib]]]{ $ProgressText.Text = $ActualElement } ),$null, $null )
            $ItemGrid.Items.Add($_)
        }
        $ProgressText.Text = " "
    })

    #Launch the window
    $xamGUI.ShowDialog() | Out-Null
        
}

function Get-ipxTreeView {
    <#
        .Synopsis
            Ermittelt alle Unterordner und optional die Dateien als Baumansicht.
        .Description
            Ermittelt alle Unterordner und optional die Dateien als Baumansicht.
            Über Dialoge werden der Suchordner und die Ergebnisdatei angegeben.
            Optional sind die zu suchende Hierarchietiefe und die Ausgabe aller Dateien.
        .Parameter Depth
            Die Ordnertiefe bis zu der gesucht werden soll.
        .Parameter ShowLeaf
            Schalter der die Ausgabe von Dateien aktiviert, Vorgabewert ist $false (nur Ordner).
        .Example 
            Get-ipxTreeview -ShowLeaf
            Fragt zuerst den Suchordner im Dialog ab, danach die Exportdatei als *.txt oder *.*
            Danach werden alle Unterordner und Dateien bis zur dritten Ebene ausgegeben.
        .Inputs
            System.IO.DirectoryInfo Suchordner als String
            System.IO.FileInfo Exportdatei als String
            System.Switch ShowLeaf (mit/ohne Dateien)
            System.Int32 Ordnertiefe
        .Outputs
            System.IO.File Exportdatei als txt oder andere Textdatei
        .Notes
            Version: 1.0.0.1
            Datum: 1.1.2016
        .Component
            Powershell Community Extension (PSCX)
            Get-ipxFoldername, Set-ipxFilename
    #>

    param (
        [Parameter(HelpMessage='Ordnertiefe, Vorgabewert 3')]
        [int32]$Depth=3,
        [Parameter(HelpMessage='Anzeige der Dateien aktivieren')]
        [switch]$ShowLeaf=$false
    )

    [string]$path = Get-ipxFoldername -rootpath G:\ -description "Ordner auswählen der durchsucht wird."
    [string]$OutFile = Set-ipxFilename -rootpath G:\ -title "Dateiname für den Export" -filter "TXT|*.txt|*|*.*"

    if ( $ShowLeaf ) {
        show-tree -Path $path -Depth $Depth -ShowLeaf | Out-File -FilePath $OutFile
    }
    else {
        show-tree -Path $path -Depth $Depth | Out-File -FilePath $OutFile
    }
}

function Split-ipxCsv {
    <#
        .Synopsis
            Teilt eine CSV-Datei in einzelne Häppchen.
        .Description
            Teilt eine CSV-Datei die zB zu groß ist in einzelne CSV auf, die Schrittweite wie viele Elemente pro CSV ausgegeben werden ist einstellbar.
            Für das Cmdlet wird eine CSV-Datei angegeben und ein Ausgabeordner, die Benamung der Ausgabedatei erfolgt anhand des Originaldateinamen und laufender Nummer.
            Das Cmdlet arbeitet mit Export-CSV im Modus -Append.
            Wird PS Core ab v7 genutzt, werden die Werte per -UseQuotes AsNeeded in die CSV geschrieben.
        .Parameter Source
            Dateiname der zu teilenden CSV-Datei (Mandatory).
        .Parameter Destination
            Ordner in dem die getielten Dateien abgelegt werden (Mandatory).
        .Parameter ValuesPerFile
            Wert wie viele Elemente pro Datei exportiert wurden, Vorgabewert 1.
        .Parameter First
            Schränkt die Verarbeitung auf die ersten angegebenen Elemente ein, Vorgabewert 0 (alles).
        .Parameter SourceDelimiter
            Trennzeichen in der CSV-Datei, Vorgabewert ;
        .Parameter DestinationDelimiter
            Trennzeichen in den gesplitteten CSV-Dateien, Vorgabewert: ;
        .Parameter Timestamp
            Gibt die Startzeit, Endzeit und Dauer für die Verarbeitung in der Shell aus.
        .Example 
            Split-ipxCsv -Source D:\input\myFile.csv -Destination D:\output -ValuesPerFile 50
            Teilt die CSV mit dem resultierenden Dateinamen myFile_0.csv myFile_50.csv ...
        .Inputs
            System.IO.FileInfo Source
            System.IO.DirectoryInfo Destination
            System.Int ValuesPerFile
            System.String Delimiter
        .Outputs
            Cmdlet in PS nichts, Dateisystem System.IO.File
        .Notes
            v1.2.0 - 07.04.2020
                + Aufteilen in SourceDelimiter und DestinationDelimiter damit das Trennzeichen austauschbar ist
            v1.1.0 - 20.12.2019
                + $First Parameter für Auswahl der ersten x Elemente
                + $Timestamp Parameter für Skriptdauer
            v1.0.0 - 18.12.2019
        .Component
            Nothing
    #>

    param (
    [Parameter(Mandatory, HelpMessage='CSV-Quelldatei')]
    [System.IO.FileInfo]$Source,
    [Parameter(Mandatory, HelpMessage='Zielordner für die extrahierten Einzelwerte')]
    [System.IO.DirectoryInfo]$Destination,
    [Parameter(HelpMessage='Werte pro separierter CSV')]
    $ValuesPerFile = 1,
    [Parameter(HelpMessage='Nur die ersten angegebenen Werte extrahieren')]
    [System.Int32]$First = 0,
    [Parameter(HelpMessage='Trennzeichen in der originalen CSV')]
    [String]$SourceDelimiter = ";",
    [Parameter(HelpMessage='Trennzeichen in den gesplitteten CSVs')]
    [String]$DestinationDelimiter = ";",
    [Parameter(HelpMessage='Zeitstempel in der shell ausgeben')]
    [switch]$Timestamp
    )

    if ( $Timestamp ) {
        $StartTime = Get-Date
        "Startzeit: {0}" -f $StartTime
    }

    $SourceCsv = Import-Csv -Path $Source.FullName -Delimiter $SourceDelimiter -Encoding UTF8

    if( $First -gt 0) {
        $SourceCsv = $SourceCsv | Select-Object -First $First
    }

    $CsvElements = $SourceCsv.Length
    $Percentage = 0

    for ($i= 0; $i -lt $SourceCsv.Length; $i=$i+$ValuesPerFile) {

        Write-Progress -Activity $i -PercentComplete ($Percentage/$CsvElements*100)
        $path = [System.String]::Format("{0}\{1}_{2}.csv", $Destination, $Source.BaseName, $i);

        for ($j = $i; $j -lt ($i + $ValuesPerFile); $j++){
            if($PSVersionTable.PSVersion.Major -lt "7"){
                $SourceCsv[$j] | Export-Csv -Path $path -Delimiter $DestinationDelimiter -NoTypeInformation -Append -Encoding UTF8 -Force -ErrorAction SilentlyContinue
            }
            else{
                $SourceCsv[$j] | Export-Csv -Path $path -Delimiter $DestinationDelimiter -NoTypeInformation -Append -Encoding UTF8 -UseQuotes AsNeeded -Force -ErrorAction SilentlyContinue
            }
        }
        $Percentage = $i
    }
    if ( $Timestamp ) {
        $EndTime = Get-Date
        "Startzeit: {0}" -f $EndTime
        "Dauer: {0}" -f ($EndTime - $StartTime)
    }
}